#
# Copyright (C) 1999-2000 VMware, Inc.
# All Rights Reserved
#
# $Id: VM.pm,v 1.27.4.1 2000/12/01 11:52:48 brettv Exp $

use VMware::Control::Server;
package VMware::Control::VM;

use strict;

require DynaLoader;

use vars qw($VERSION @ISA);

$VERSION = '0.02';

@ISA = qw(DynaLoader);

use VMware::Control '0.02';
#bootstrap VMware::Control $VERSION;

# Preloaded methods go here.

sub new {
    my $server = shift;
    my $vm_id = shift;
    return New($server, $vm_id);
}

sub get_last_error {
    my $vm = shift;
    my ($ret, $string) = $vm->GetLastError();
    if (wantarray) {
        return ($ret, $string);
    } else {
        return $ret;
    }
}

sub is_connected {
    my $vm = shift;
    return $vm->IsConnected();
}

sub connect {
    my $vm = shift;
    my $ret = $vm->Connect();
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub start {
    my $vm = shift;
    my $ret = $vm->Start();
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub stop {
    my $vm = shift;
    my $force = shift;
    if (!defined($force)) {
        $force = 0;
    }
    my $ret = $vm->Stop($force);
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub request_stop {
    my $vm = shift;
    my $force = shift;
    if (!defined($force)) {
        $force = 0;
    }
    my $ret = $vm->RequestStop($force);
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub reset {
    my $vm = shift;
    my $force = shift;
    if (!defined($force)) {
        $force = 0;
    }
    my $ret = $vm->Reset($force);
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub request_reset {
    my $vm = shift;
    my $force = shift;
    if (!defined($force)) {
        $force = 0;
    }
    my $ret = $vm->RequestReset($force);
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub suspend_to_disk {
    my $vm = shift;
    my $ret = $vm->SuspendToDisk();
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub info {
    my $vm = shift;
    my @list = $vm->Info();
    my %hash;
    my ($k, $v);

    if ($#list <= 0) {
        return undef;
    }

    if (! ($#list % 2)) {
        print STDERR "VMware::Control::VM::info: Warning: hash has odd number of elements\n";
    }

    keys(%hash) = $#list;
    while($#list > 0) {
        ($k, $v) = splice(@list, 0, 2);
        $hash{$k} = $v;
    }
    return \%hash;
}

sub disconnect {
    my $vm = shift;
    my $ret = $vm->Disconnect();
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub get ($$@) {
    my $vm = shift;
    my $var = shift;
    if ( $var =~ m/^Status\.stats\./ ) {
        my $ret;

        my ($units, $allocation, $limit, @values);

        if ($#_ >= 0) {
            $var .= "[" . shift;
            while($_ = shift) {
                $var .= "," . $_;
            }
            $var .= "]";
        }

        #
        # Status.stats.*
        #
        if ($var =~ m/^Status\.stats\.([^\.]+)\.([^\.]+)\[(.+)\]$/ ) {
            my $interval_string = $3;
            my $device = $1;
            my $statname = $2;
            my @intervals;

            while ($interval_string =~ s/,*([^,]+)//) {
                push(@intervals, $1);
            }


            #print STDERR "  Perl API debug Get  device=", $device, 
            #" statname=", $statname, " intervals=", @intervals,"\n";

            $ret = $vm->GetAsString($var);

            #print STDERR "  Perl API debug Get(",$var,") = ", $ret,"\n";

            if (!defined($ret) || $ret =~ /^Error:/) {
                $vm->SetError(VM_E_NOPROPERTY, $ret);
                return undef;
            }

            # Convert a list of the form functions[:values]+
            # to a hash
            if ($ret =~ s/^([^:]+)://) {
                my @functions = split(/,/, $1);
                my %h;

                my @value_lists = split (/:/, $ret);
            
                for my $valuelist (@value_lists) {
                    my @values = split (/,/, $valuelist);
                    my $interval = shift (@intervals);
                    foreach my $function (@functions) {
                        my $ref = $h{$function}; #Reference to a hash
                        my %fvalues;
                        if (defined($ref)) {
                            %fvalues = %{$h{$function}};
                        }
                        $fvalues{$interval} = shift(@values);
                        $h{$function} =  \%fvalues; 
                    }
                }

                return \%h;
            } else {
                #Error
                print STDERR "VMware::Control::VM::get: Invalid property $var: ", $ret, "\n";
                $vm->SetError(VM_E_NOPROPERTY, $ret);
                return undef;
            }
        }

        $ret = $vm->GetAsString($var);

        #print STDERR "  Perl API debug Get(",$var,") = ", $ret,"\n";

        if (!defined($ret) || $ret =~ /^Error:/) {
          $vm->SetError(VM_E_NOPROPERTY, $ret);
          return undef;
        } elsif ($var =~ /\.devices$/ || $var =~ /\.stats$/) {
            my @retlist = split (/,/, $ret);
            return \@retlist;
        } elsif ($var =~ /\.info$/) {
            my %h;
            while ($ret =~ s/^([^:]+):([^:]*):*//) {
                if ($1 eq "functions") {
                    my @l = split(/,/, $2);
                    $h{$1} = \@l;
                } else {
                    $h{$1} = $2;
                }
            }
            return \%h;
        } else {
            return $ret;
        }
    } else {
        my $ret = $vm->GetAsString($var);

        if (!defined($ret)) {
            return undef;
        } elsif ($var eq "Status.devices") {
            my @retlist = split (/,/, $ret);
            return \@retlist;
        } else {
            return $ret;
        }
    }
}

sub answer_question {
    my $vm = shift;
    my $seq = shift;
    my $choicen = shift;
    my $ret = $vm->AnswerQuestion($seq, $choicen);

    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

# Convert a list of the form functions[:values]+
# to a hash.  If a value has a command (",") in it,
# it is split into a list itself
sub string_to_complex_hash {
    my $string = shift;
    my @pieces = split(/:/, $string);
    my %hash;

    if (!$#pieces) {
        return undef;
    }

    if (! ($#pieces % 2)) {
        print STDERR "VMware::Control::VM Error: hash has odd number of elements\n";
        return undef;
    }

    while( $#pieces > 0) {
        my $key = shift (@pieces);
        my $value = shift (@pieces);

        #print STDERR "Debug: $key = $value\n";

        my @sub_pieces = split(/,/, $value);
        if ($#sub_pieces) {
            $hash{$key} = \@sub_pieces;
        } else {
            $hash{$key} = $value;
        }
    }

    return %hash;
}

#Initialize the VM portion API library and return 1 if OK
VMware::Control::VM::Init();

__END__

=head1 NAME

VMware::Control::VM - A perl module for controlling VMware virtual machines.

=head1 SYNOPSIS

  use VMware::Control::VM;

  $vm = VMware::Control::VM::new($server, $vm_identifier);
  $err = $vm->connect();
  if (!defined($err)) {
    ($errorNumber, $errorString) = $vm->get_last_error();
  }

  $ret = $vm->start();

  $ret = $vm->stop();
  $ret = $vm->stop(1);

  $ret = $vm->reset();

  $ret = $vm->suspend_to_disk();

  %info = %{ $vm->info() };

  $value = $vm->get("Status.power");

  $value = $vm->get("Config.log.filename");

  $vm->disconnect();

=head1 DESCRIPTION

This packages provides an interface for interacting with a VMware
virtual machine.  Information about the configuration and status of
the machine can be queried.  

CAUTION: This API is preliminary and will change significantly between
this Beta and the final release version.

=item VMware::Control::VM::new($server, $vm_id)

Establish a connection to the specified VM on the specified server.
Valid VM_IDENTIFIERs can be obtained using the
VMware::Control::Server::enumerate method.

=item $vm->connect()

Attempt to establish a connection to the specified VM on the server.

=item $vm->get_last_error()

When a method fails (that is, returns undef), this method will return
the error code corresponding to the failure, along with a string
description (often with more specific error information).

=item $vm->start()

Start (power on) the VM.

=item $vm->stop($force)

Stop (power off) the VM.  If $force is specified and true, perform
hard power-off rather than attempting a soft (clean) shutdown of the
guest OS.

=item $vm->reset($force)

Reset (power off and on) the VM.  If $force is specified and true,
perform hard reset rather than attempting a soft (clean) reboot of the
guest OS.

=item $vm->suspend_to_disk()

Suspend the VM to disk.

=item $vm->info()

Given a VM identifier, this allows us to get more detailed information
on the VM.

This function returns a list of property-value pairs (that is, a hash)
so as to be self describing and extensible.  An example return value
might be C<("Config.filename", "biff.cfg", "Config.name", "biff",
"Config.guestOS", "Novell", "Config.memorySize", 512)>.  Nothing is
available in the C<info> data that isn't available via the C<get>
method, it's just a little simpler to use C<info>.

[Eventually, the set of information returned by C<info> will be
customizable by modifying the C<Config.infoValues> property.]

The C<info> method is designed to simplify quick status summaries.

Returns a list of property values pairs.

=item $vm->get(PROPERTY_NAME)

Get a property from a VM.  

=item $vm->disconnect()

This disconnects from the remote Virtual Machine.

=head1 Name space for Properties

The name space for properties is divided into four top-level domains.
The C<Config.*> domain contains all settings and parameters and is
logically equivalent to the configuration database.  The
C<ServerStatus.*> and C<Status.*> domains contain status and
performance information for the server and specific VMs respectively.
The C<Server.*> domain describes the global configuration information
for the server.

=head2 Variables

The C<Config.*> name space models the existing configuration database.
For example, C<$vm-E<gt>get("Config.ethernet0.present", $foo)> would
set $foo to TRUE if ethernet0 was present.

=head2 Status

The C<Status.*> name space returns information on the state of the VM
and also performance information.  The C<ServerStatus.*> name space
returns information on the state of the server and also performance
information.  Both name spaces are read-only.

A lot more definition work will happen here, but as an example,
C<Status.power> has a value in the set C<("on", "off", "suspended")>.
Performance and statistical information is also available via the
C<Status.stats.*> namespace, although this is not yet well defined.

=head1 AUTHORS

John Haggerty, Daniel Kerns, Brett Vasconcellos.

=head1 COPYRIGHT

    (c) 2000 VMware Incorporated.  All rights reserved.

=head1 VERSION

Version 0.2	   21 Sep  2000

C<% pod2html --header --title="VMware::Control::Server" E<lt> Server.pm E<gt> perlAPI-server.html>

=head1 SEE ALSO

perl(1).

=cut
